==> pitaya-framework-0.0.9.tgz: package/src/view.ts <==
import { NavigationInstruction, RedirectToRoute, RouteConfig }          from "aurelia-router";
import { IPitayaViewSettings, IScrollMarkerHandlerConfig, IViewStates } from "../interfaces.d";
import { navigate, PitayaApp, setStatusBarColor }                       from "./app";
import { PitayaAppConfiguration }                                       from "./app-configuration";
import { PitayaViewCustomElement }                                      from "./components/pitaya-view";
import { log }                                                          from "./logger";

debugger;

const _CLASS_ANIMATION_READY: string  = "pitaya-view--animation-ready";
const _VIEW_ELEMENT_NAME: string      = "pitaya-view";
const _SUB_VIEW_ELEMENT_NAME: string  = "pitaya-sub-view";
const _TABS_VIEW_ELEMENT_NAME: string = "pitaya-tabs-view";
const _PITAYA_PAGES_CLASS: string     = "au-leave-active";
const _ANIMATED_CALLBACK_NAME: string = "animated";

const _VIEW_STATUS_PRESET: IViewStates = {
	active   : false,
	created  : false,
	bound    : false,
	attached : false,
	animated : false,
	detached : false
};

const _MDC_COLOR_VARIABLE_MAP = {
	"primary"      : "--mdc-theme-primary",
	"secondary"    : "--mdc-theme-secondary",
	"on-primary"   : "--mdc-theme-on-primary",
	"on-secondary" : "--mdc-theme-on-secondary",
	"background"   : "--mdc-theme-background",
	"on-surface"   : "--mdc-theme-on-surface",
	"surface"      : "--mdc-theme-surface",
};

let _currentViewInstance: PitayaView = null;

export abstract class PitayaView
{
	static get instance()
	{
		return _currentViewInstance;
	}

	private _viewModel: PitayaViewCustomElement;
	private _animationTaskQueue: Function[] = [];
	private _settings: IPitayaViewSettings  = {
		isPublic  : true,
		hierarchy : 0 as number,
		theme     : PitayaAppConfiguration.View.defaultTheme
	};

	private _rootElement: HTMLElement;

	get rootElement(): HTMLElement
	{
		return this._rootElement;
	}

	private _states: IViewStates = Object.assign( {}, _VIEW_STATUS_PRESET );

	get states(): IViewStates
	{
		return this._states;
	}

	private _innerElement: HTMLElement;

	get innerElement(): HTMLElement
	{
		return this._innerElement;
	}

	/**
	 * @aurelia hook
	 */
	public canActivate( params: any, routeConfig: RouteConfig, navigationInstruction: NavigationInstruction ): boolean | RedirectToRoute | Promise<boolean | RedirectToRoute>
	{
		const { View, Development } = PitayaAppConfiguration;

		if( this._settings.isPublic
			|| ( Development.enabled && Development.disablePrivateRedirects )
			|| !View.privateRedirectView )
		{
			return true;
		}

		return new RedirectToRoute( View.privateRedirectView );
	}

	/**
	 * @aurelia hook
	 */
	public activate( params, routeConfig, navigationInstruction )
	{
		this._settings.hierarchy = routeConfig.nav || 0;

		this._states.active = true;
	}

	/**
	 * @aurelia hook
	 */
	public created( parentContext, context )
	{
		this._states.created = true;

		const model = context.controllers.find( ( controller ) =>
		{
			const { elementName } = controller.behavior;

			return elementName === _VIEW_ELEMENT_NAME
				|| elementName === _SUB_VIEW_ELEMENT_NAME
				|| elementName === _TABS_VIEW_ELEMENT_NAME;
		} );

		this._viewModel = model.viewModel;

		_currentViewInstance = this;
	}

	/**
	 * @aurelia hook
	 */
	public bind( bindingContext, overrideContext )
	{
		this._states.bound = true;
	}

	/**
	 * @aurelia hook
	 */
	public attached()
	{
		this._states.attached = true;
		this._rootElement     = this._viewModel.rootElement as HTMLElement;
		this._innerElement    = this._viewModel.innerElement;

		this._setAnimationListener();
	}

	/**
	 * @aurelia hook
	 */
	public detached()
	{
		this._states.attached = false;
		this._states.detached = true;
	}

	public async navigate( route: string, options?: any )
	{
		return navigate( route, options );
	}

	public queueAnimationTask( task: Function )
	{
		this._animationTaskQueue.push( task );
	}

	public setTheme( color: string = this._settings.theme )
	{
		setStatusBarColor( _getColorFromMdcVariable( color ) );
	}

	private _willBeAnimated(): boolean
	{
		const { rootElement } = this;
		// @ts-ignore
		const siblings        = [ ...rootElement.parentElement.children ];

		return siblings.some( ( sibling ) => sibling !== rootElement && sibling.classList.contains( _PITAYA_PAGES_CLASS ) );
	}

	private async _setAnimationListener()
	{
		const { rootElement } = this;

		if( !this._willBeAnimated() )
		{
			this._onAnimationEnd();

			return;
		}

		rootElement.addEventListener( "animationend", this._onAnimationEnd.bind( this ), { once : true } );
	}

	private async _onAnimationEnd()
	{
		const queue = this._animationTaskQueue;

		while( queue.length )
		{
			await queue.pop()();
		}

		this.rootElement.classList.add( _CLASS_ANIMATION_READY );

		this._states.animated = true;

		this[ _ANIMATED_CALLBACK_NAME ] && this[ _ANIMATED_CALLBACK_NAME ]();
	}
}

export const scrollToTop = function( scrollDuration: number ): void
{
	const scrollStep     = -window.scrollY / ( scrollDuration / 15 );
	const scrollInterval = setInterval( function()
	{
		if( window.scrollY !== 0 )
		{
			window.scrollBy( 0, scrollStep );
		}
		else
		{
			clearInterval( scrollInterval );
		}
	}, 15 );
};

export const setScrollMarkerHandler = function( { element, className, minHeight, maxHeight }: IScrollMarkerHandlerConfig ): () => void
{
	const body               = document.body;
	const documentElement    = document.documentElement;
	const viewHeight: number = Math.max( documentElement.clientHeight, window.innerHeight || 0 );

	minHeight = minHeight === undefined ? 0 : minHeight;
	maxHeight = maxHeight === undefined ? viewHeight : maxHeight;

	const setScrollMarkerClass = function()
	{
		const scrollTop   = Math.max( body.scrollTop, documentElement.scrollTop );
		const addOrRemove = scrollTop >= minHeight && scrollTop <= maxHeight ? "add" : "remove";

		element.classList[ addOrRemove ]( className );
	};

	setScrollMarkerClass();

	window.addEventListener( "scroll", setScrollMarkerClass );

	return () =>
	{
		window.removeEventListener( "scroll", setScrollMarkerClass );
	};
};

const _getColorFromMdcVariable = function( color: string ): string
{
	if( color.startsWith( "#" ) || color.startsWith( "rgb" ) )
	{
		return color;
	}

	if( !( color in _MDC_COLOR_VARIABLE_MAP ) )
	{
		log.error( "view.js _getColorFromMdcVariable()", `No MDC variable found that matches '${color}'` );

		return "";
	}

	return getComputedStyle( PitayaApp.instance.appView.rootElement ).getPropertyValue( _MDC_COLOR_VARIABLE_MAP[ color ] );
};

==> pitaya-framework-0.0.10.tgz: package/src/view.ts <==
import { NavigationInstruction, RedirectToRoute, RouteConfig }          from "aurelia-router";
import { IPitayaViewSettings, IScrollMarkerHandlerConfig, IViewStates } from "../interfaces.d";
import { navigate, PitayaApp, setStatusBarColor }                       from "./app";
import { PitayaAppConfiguration }                                       from "./app-configuration";
import { PitayaViewCustomElement }                                      from "./components/pitaya-view";
import { log }                                                          from "./logger";

const _CLASS_ANIMATION_READY: string  = "pitaya-view--animation-ready";
const _VIEW_ELEMENT_NAME: string      = "pitaya-view";
const _PITAYA_PAGES_CLASS: string     = "au-leave-active";
const _ANIMATED_CALLBACK_NAME: string = "animated";

const _VIEW_STATUS_PRESET: IViewStates = {
	active   : false,
	created  : false,
	bound    : false,
	attached : false,
	animated : false,
	detached : false
};

const _MDC_COLOR_VARIABLE_MAP = {
	"primary"      : "--mdc-theme-primary",
	"secondary"    : "--mdc-theme-secondary",
	"on-primary"   : "--mdc-theme-on-primary",
	"on-secondary" : "--mdc-theme-on-secondary",
	"background"   : "--mdc-theme-background",
	"on-surface"   : "--mdc-theme-on-surface",
	"surface"      : "--mdc-theme-surface",
};

let _currentViewInstance: PitayaView = null;

export abstract class PitayaView
{
	static get instance()
	{
		return _currentViewInstance;
	}

	private _viewModel: PitayaViewCustomElement;
	private _animationTaskQueue: Function[] = [];
	private _settings: IPitayaViewSettings  = {
		isPublic  : true,
		hierarchy : 0 as number,
		theme     : PitayaAppConfiguration.View.defaultTheme
	};

	private _rootElement: HTMLElement;

	get rootElement(): HTMLElement
	{
		return this._rootElement;
	}

	private _states: IViewStates = Object.assign( {}, _VIEW_STATUS_PRESET );

	get states(): IViewStates
	{
		return this._states;
	}

	private _innerElement: HTMLElement;

	get innerElement(): HTMLElement
	{
		return this._innerElement;
	}

	/**
	 * @aurelia hook
	 */
	public canActivate( params: any, routeConfig: RouteConfig, navigationInstruction: NavigationInstruction ): boolean | RedirectToRoute | Promise<boolean | RedirectToRoute>
	{
		const { View, Development } = PitayaAppConfiguration;

		if( this._settings.isPublic
			|| ( Development.enabled && Development.disablePrivateRedirects )
			|| !View.privateRedirectView )
		{
			return true;
		}

		return new RedirectToRoute( View.privateRedirectView );
	}

	/**
	 * @aurelia hook
	 */
	public activate( params, routeConfig, navigationInstruction )
	{
		this._settings.hierarchy = routeConfig.nav || 0;

		this._states.active = true;
	}

	/**
	 * @aurelia hook
	 */
	public created( parentContext, context )
	{
		this._states.created = true;

		const model = context.controllers.find( ( controller ) =>
		{
			const { elementName } = controller.behavior;

			return elementName === _VIEW_ELEMENT_NAME;
		} );

		this._viewModel = model.viewModel;

		_currentViewInstance = this;
	}

	/**
	 * @aurelia hook
	 */
	public bind( bindingContext, overrideContext )
	{
		this._states.bound = true;
	}

	/**
	 * @aurelia hook
	 */
	public attached()
	{
		this._states.attached = true;
		this._rootElement     = this._viewModel.rootElement as HTMLElement;
		this._innerElement    = this._viewModel.innerElement;

		this._setAnimationListener();
	}

	/**
	 * @aurelia hook
	 */
	public detached()
	{
		this._states.attached = false;
		this._states.detached = true;
	}

	public async navigate( route: string, options?: any )
	{
		return navigate( route, options );
	}

	public queueAnimationTask( task: Function )
	{
		this._animationTaskQueue.push( task );
	}

	public setTheme( color: string = this._settings.theme )
	{
		setStatusBarColor( _getColorFromMdcVariable( color ) );
	}

	private _willBeAnimated(): boolean
	{
		const { rootElement } = this;
		// @ts-ignore
		const siblings        = [ ...rootElement.parentElement.children ];

		return siblings.some( ( sibling ) => sibling !== rootElement && sibling.classList.contains( _PITAYA_PAGES_CLASS ) );
	}

	private async _setAnimationListener()
	{
		const { rootElement } = this;

		if( !this._willBeAnimated() )
		{
			this._onAnimationEnd();

			return;
		}

		rootElement.addEventListener( "animationend", this._onAnimationEnd.bind( this ), { once : true } );
	}

	private async _onAnimationEnd()
	{
		const queue = this._animationTaskQueue;

		while( queue.length )
		{
			await queue.pop()();
		}

		this.rootElement.classList.add( _CLASS_ANIMATION_READY );

		this._states.animated = true;

		this[ _ANIMATED_CALLBACK_NAME ] && this[ _ANIMATED_CALLBACK_NAME ]();
	}
}

export const scrollToTop = function( scrollDuration: number ): void
{
	const scrollStep     = -window.scrollY / ( scrollDuration / 15 );
	const scrollInterval = setInterval( function()
	{
		if( window.scrollY !== 0 )
		{
			window.scrollBy( 0, scrollStep );
		}
		else
		{
			clearInterval( scrollInterval );
		}
	}, 15 );
};

export const setScrollMarkerHandler = function( { element, className, minHeight, maxHeight }: IScrollMarkerHandlerConfig ): () => void
{
	const body               = document.body;
	const documentElement    = document.documentElement;
	const viewHeight: number = Math.max( documentElement.clientHeight, window.innerHeight || 0 );

	minHeight = minHeight === undefined ? 0 : minHeight;
	maxHeight = maxHeight === undefined ? viewHeight : maxHeight;

	const setScrollMarkerClass = function()
	{
		const scrollTop   = Math.max( body.scrollTop, documentElement.scrollTop );
		const addOrRemove = scrollTop >= minHeight && scrollTop <= maxHeight ? "add" : "remove";

		element.classList[ addOrRemove ]( className );
	};

	setScrollMarkerClass();

	window.addEventListener( "scroll", setScrollMarkerClass );

	return () =>
	{
		window.removeEventListener( "scroll", setScrollMarkerClass );
	};
};

const _getColorFromMdcVariable = function( color: string ): string
{
	if( color.startsWith( "#" ) || color.startsWith( "rgb" ) )
	{
		return color;
	}

	if( !( color in _MDC_COLOR_VARIABLE_MAP ) )
	{
		log.error( "view.js _getColorFromMdcVariable()", `No MDC variable found that matches '${color}'` );

		return "";
	}

	return getComputedStyle( PitayaApp.instance.appView.rootElement ).getPropertyValue( _MDC_COLOR_VARIABLE_MAP[ color ] );
};
